--  <prjroot>/cp/draw/singlelinetext.lua

--[[    SingleLineText      Function

    (revised from original function, 2013-Dec-12 dD)

    Blits a single line of text to screen with optional text scrolling
    in the horizontal plane.  If scrolling is required, an anim pointer
    is returned.

    Accepts a signed, foating-point leadingFactor to move the text up or down within the frame.

    Padding sets the internal boundRect on all four sides.

    Values for align:   left, right, center

--]]

require( "format.stringtrunc" )
--require( "skins.position" )

function
        SingleLineText(     textString,
                            fontID,

                            frameLeftXpos,
                            frameTopYpos,
                            frameWidth,
                            frameHeight,

                            leadingFactor,
                            align,
                            padding,

                            enableScrolling,
                            wipeBounds )

    local   textSurface = nil
    local   size = { w = 0, h = 0 }

    local   requiresScrolling = nil
    local   animRect = nil
    local   anim = nil

    local   pctSize = 0        
    local   truncTextString = nil
    local   truncChars = 0
    local   wipeBounds = wipeBounds or false

    local   xOffset, yOffset = 0, 0

    local   boundLeftXpos = frameLeftXpos + padding
    local   boundTopYpos = frameTopYpos + padding
    local   boundWidth = frameWidth - (2 * padding)
    local   boundHeight = frameHeight - (2 * padding)

    --  RenderText to textSurface and getSize  

    textSurface =   Display.RenderText( fontID, textString )
    size.w, size.h = textSurface:getSize()

    --  Create a Y offset from leading

    yOffset = math.ceil( leadingFactor * size.h )


    --  Evaluate the need for scrolled text

    if      size.w <= boundWidth
    then
        requiresScrolling = false
    else
        requiresScrolling = true
        pctSize = (size.w / boundWidth)
    end

    --  Scroll, truncate or do nothing unusual

    local config = GetConfig()
    local scroll = tonumber(config:Get("text_display", "0"))
    local continuous = true

    if scroll < 2 then
        if scroll == 1 then
            continuous = false
        end
    else
        enableScrolling = false
    end

    if      requiresScrolling
    then
        if  enableScrolling
        then      
            --  Generate anim for scrolling

            animRect = SDL.newRect( boundLeftXpos,
                                    boundTopYpos,
                                    boundWidth,
                                    boundHeight )
            anim = Display.AcquireScrollingText(textSurface,
                                                animRect,
                                                continuous,
                                                1.0, 30 )
            

            return  anim, boundWidth, boundHeight

        else
            --  Truncate the string ...

            textSurface = nil

            truncChars = math.floor( #textString / ( pctSize ))
            truncTextString = string.gsub(StringTrunc(textString, truncChars - 4,"",""),"\n.*", "") .. " ..."

            --  Generate truncated textSurface  

            textSurface =   Display.RenderText( fontID, 
                                                truncTextString )
            size.w, size.h = textSurface:getSize()
        end        
    else 
        --  Do nothing unusual
    end
    --  Center text, clip to bounds
    
    if      align == "center"
    then
        xOffset = math.floor(
            (boundWidth - size.w) / 2)
        if xOffset < 0 then
            xOffset = 0
        end
        yOffset = math.floor(( boundHeight 
                                   - ( size.h * 
                                           ( 1 + leadingFactor ))) / 2)
        if yOffset < 0 then
            yOffset = 0
        end
    elseif  align == "right"
    then
        xOffset = math.floor( boundWidth - size.w )
    else
        xOffset = 0
    end
    
    --  For truncated or normal, create text frame and clipping box

    local   boundRect = SDL.newRect(boundLeftXpos + xOffset,  
                                    boundTopYpos + yOffset,
                                    size.w, size.h )

    local   clipRect = SDL.newRect( 0, 0, size.w, size.h )

    --  Wipe bounds

    if      wipeBounds
    then
        Display.DrawRectangleFilled(
            boundLeftXpos + xOffset,  
            boundTopYpos + yOffset,
            size.w, 
            size.h,
            red )
    end

    --  Draw text to frame
    SDL.BlitSurface( textSurface, clipRect, screen, boundRect )
    return nil, size.w, size.h
end







