from flask import Flask, jsonify, current_app, Blueprint, render_template 
from flask_restful import Resource, Api, reqparse
from flask_swagger import swagger
import json

import python_settings
from preampsetting import LockCFunction
from pylogger import bllogger

import libpreamp_lib as amp
import time

api = Api()
api_point = Blueprint('api', __name__)

def _marshal_device_table(devices):
    rtn = {'device_table':[]}
    for i in devices:
        device = i.data() 
        rtn['device_table'].append({'name':device.deviceName, 'mac':device.deviceMAC})
    return rtn 

def _marshal_input(input_list):
    inputs = {'inputs':[]}
    input_res = python_settings.preamp_setting.preamp_setting.GetInputResolution()
    for i ,input in enumerate(input_list):
        inputs['inputs'].append({'name':input.name, 'trim':input.trim * 0.1, 'polarity': input.polarity, 'index': input.index,
                        'balance': input.balance * 0.1, 'theater_mode': input.theater_mode, 
                        'recordable': input.recordable})
    return inputs
    

class Input(Resource):
    @LockCFunction
    def get(self):
        """
	Get a list of the inputs
        
        ---
        tags:
          - Input
        responses:
          200:
            description: Returns a list of inputs
        """
	return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
	inputs = []
	for i in range(6):
	    input = python_settings.preamp_setting.preamp_setting.GetInputDevice(i)
	    inputs.append(input)
	return _marshal_input(inputs)

    @LockCFunction 
    def post(self):
        """
	Get a list of the inputs
        
        ---
        swagger_from_file: website/swagger/input_device_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('index', type=int, location='args')
	parser.add_argument('name', type=str, default=None, location='args')
	parser.add_argument('trim', type=float, default=None, location='args')
	parser.add_argument('balance', type=float, default=None, location='args')
	parser.add_argument('polarity', type=str, default=None, location='args')
	parser.add_argument('theater_mode', type=str, default=None, location='args')
	parser.add_argument('recordable', type=str, default=None, location='args')
	args = parser.parse_args()
        input = python_settings.preamp_setting.preamp_setting.GetInputDevice(args.index)
        if not args['polarity'] is None:
            args['polarity'] = True if args['polarity'].lower()=='true' else False
        if not args['theater_mode'] is None:
            args['theater_mode'] = True if args['theater_mode'].lower()=='true' else False
        if not args['recordable'] is None:
            args['recordable'] = True if args['recordable'].lower()=='true' else False
        for attr in args:
            if args[attr] is None:
                args[attr] = getattr(input, attr)
                if attr == 'trim':
                    args[attr] *= 0.1
	inputs = []
	python_settings.preamp_setting.preamp_setting.SetInputDevice( int(args['index']), str(args['name']), int( args['trim']*10), int(args['balance']*10),  args['polarity'], args['theater_mode'], args['recordable'])
        return {'input':{'name':args['name'], 'trim':args['trim'], 'index': args['index'], 'balance': args['balance'], 'polarity': args['polarity'], 
            'theater_mode':args['theater_mode'], 'recordable': args['recordable']}}

class CurrentInput(Resource):
    @LockCFunction
    def get(self):
        """
	Get the index of the current input
        
        ---
        tags:
          - Input
        responses:
          200:
            description: Returns the index of the current input
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        index = python_settings.preamp_setting.preamp_setting.GetCurrentInput()
        return {'current_input': index}

    @LockCFunction
    def post(self):
        """
	Set the current index
        
        ---
        swagger_from_file: website/swagger/current_input_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('index', type=int, location='args')
        args = parser.parse_args()
        python_settings.preamp_setting.preamp_setting.SetCurrentInput(args['index'])
        return jsonify({'current_input': args['index']})

class InputDisabled(Resource):
    @LockCFunction
    def get(self):
        """
	Get if the input is disabled
        
        ---
        tags:
          - Input
        responses:
          200:
            description: Returns a bool saying whether the input is disabled
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        input_disabled = python_settings.preamp_setting.preamp_setting.GetInputDisabled()
        return {'input_disabled': input_disabled}

    @LockCFunction
    def post(self):
        """
        Disable/Enable inputs
        ---
        tags:
          - Input
        parameters:
          - in: query
            name: input_disabled
            schema:
              id: Input
              required:
                - input_disabled
              properties:
                balance:
                  type: str
                  description: Set to true of false, whether the input is disabled or not
        responses:
          201:
            description: The inputs have been successfully enabled/disabled 
        """ 
	parser = reqparse.RequestParser()
	parser.add_argument('input_disabled', type=str, location='args')
        args = parser.parse_args()
        disabled = True if args['input_disabled'].lower()=='true' else False
        python_settings.preamp_setting.preamp_setting.SetInputDisabled(disabled)
        return jsonify({'input_disabled': disabled})


class CurrentAux(Resource):
    @LockCFunction
    def get(self):
        """
	Get the index of the current aux
        
        ---
        tags:
          - Input
        responses:
          200:
            description: Returns the index of the current input
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        index = python_settings.preamp_setting.preamp_setting.GetCurrentAux()
        return {'current_aux': index}


    @LockCFunction
    def post(self):
        """
	Set the current index
        
        ---
        swagger_from_file: website/swagger/current_aux_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('index', type=int, location='args')
        args = parser.parse_args()
        python_settings.preamp_setting.preamp_setting.SetCurrentAux(args['index'])
        return jsonify({'current_aux': args['index']})


class AuxDisabled(Resource):
    @LockCFunction
    def get(self):
        """
	Get the state of the aux
        
        ---
        tags:
          - Input
        responses:
          200:
            description: Returns true if the aux is disabled false otherwise
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        aux_disabled = python_settings.preamp_setting.preamp_setting.GetAuxDisabled()
        return {'aux_disabled': aux_disabled}

    @LockCFunction
    def post(self):
        """
	Disable or enable the aux input
        
        ---
        swagger_from_file: website/swagger/current_aux_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('aux_disabled', type=str, location='args')
        args = parser.parse_args()
        disabled = True if args['aux_disabled'].lower()=='true' else False
        python_settings.preamp_setting.preamp_setting.SetAuxDisabled(disabled)
        return jsonify({'aux_disabled': disabled})


class RoomName(Resource):
    @LockCFunction
    def get(self):
        """
        Get the room name 
        
        ---
        tags:
          - Room
        responses:
          200:
            description: Returns the room name
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        room_name = python_settings.preamp_setting.room
        return {'room_name': room_name}

    @LockCFunction
    def post(self):
        """
        Create a new user
        ---
        swagger_from_file: website/swagger/room_name_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('room_name', type=str, location='args')
        args = parser.parse_args()
        room_name = args['room_name']
        python_settings.preamp_setting.set_room(room_name)
        return jsonify({'room_name': room_name})

class Balance(Resource):
    @LockCFunction
    def get(self):
        """
        Get the balance in dB 
        
        ---
        tags:
          - Balance
        responses:
          200:
            description: Returns the balance in dB
        """
	return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
	balance = python_settings.preamp_setting.preamp_setting.GetBalance() * 0.1
	return {'balance': balance}

    @LockCFunction
    def post(self):
        """
        Change the balance, takes number in decibells
        ---
        swagger_from_file: website/swagger/balance_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('balance', type=float, location='args')
	balance = parser.parse_args()['balance'] * 10
	python_settings.preamp_setting.preamp_setting.SetBalance(int(balance))
	return jsonify({'balance': balance})


class Blend(Resource):
    @LockCFunction
    def get(self):
        """
	Get the blend in dB
        
        ---
        tags:
          - Blend
        responses:
          200:
            description: Returns the set blend 
        """
	return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
	blend = python_settings.preamp_setting.preamp_setting.GetBlend()
	return {'blend': blend}

    @LockCFunction
    def post(self):
        """
        Change the blend, takes number in decibells
        ---
        swagger_from_file: website/swagger/blend_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('blend',type=float, location='args')
	blend = parser.parse_args()['blend']
	python_settings.preamp_setting.preamp_setting.SetBlend(int(blend))
	return jsonify({'blend': blend})

class VolumeResolution(Resource):
    @LockCFunction
    def get(self):
        """
	Get the master volume resolution
        tags:
          - VolumeResolution
        responses:
          200:
            description: Returns the set volume resolution 
        """
	return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
	volume_resolution = python_settings.preamp_setting.preamp_setting.GetVolumeResolution() * 0.1
	return {'volume_resolution': volume_resolution}

    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/volume_resolution_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('volume_resolution', type=float, location='args')
	args = parser.parse_args()
	res = args['volume_resolution']
	if res==0.1:
	    volume_resolution = amp.VolumeResolution.RES_0_1
	elif res==0.5:
	    volume_resolution = amp.VolumeResolution.RES_0_5
	elif res==1:
	    volume_resolution = amp.VolumeResolution.RES_1_0
	else:
	    return jsonify({'volume_resolution': -1})
	python_settings.preamp_setting.preamp_setting.SetVolumeResolution(volume_resolution)

	test_res = python_settings.preamp_setting.preamp_setting.GetVolumeResolution()
	print test_res
	return jsonify({'volume_resolution': res})


class MasterVolume(Resource):
    @LockCFunction
    def get(self):
        """
	Get the master volume in dB
        
        ---
        tags:
          - MasterVolume
        responses:
          200:
            description: Returns the set master volume 
        """
	return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
	master_volume = python_settings.preamp_setting.preamp_setting.GetMasterVolume() 
	master_volume = python_settings.preamp_setting.preamp_setting.ScaleMasterVolume(master_volume) * 0.1
	return {'master_volume': master_volume}

    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/master_vol_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('master_volume', type=float, location='args')
	master_vol = parser.parse_args()['master_volume'] * 10
        master_vol = python_settings.preamp_setting.preamp_setting.DescaleMasterVolume(int(master_vol))
	python_settings.preamp_setting.preamp_setting.SetMasterVolume(int(master_vol))
	return jsonify({'master_volume': master_vol})

class DefaultVolume(Resource):
    @LockCFunction
    def get(self):
        """
	Get the Default Volume in dB
        
        ---
        tags:
          - Volume
        responses:
          200:
            description: Returns the set Default Volume
        """
	return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
	default_volume = python_settings.preamp_setting.preamp_setting.GetDefaultVolume() * 0.1
	return {'default_volume': default_volume}

    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/default_vol_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('default_volume', type=float, location='args')
	default_volume = parser.parse_args()['default_volume'] * 10
	python_settings.preamp_setting.preamp_setting.SetDefaultVolume(int(default_volume))
	return jsonify({'default_volume': default_volume})

class MaxVolume(Resource):
    @LockCFunction
    def get(self):
        """
	Get the Max volume in dB
        
        ---
        tags:
          - Volume
        responses:
          200:
            description: Returns the set Max volume 
        """
	return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
	max_volume = python_settings.preamp_setting.preamp_setting.GetMaxVolume() * 0.1
	return {'max_volume': max_volume}

    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/max_vol_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('max_volume', type=float, location='args')
	max_volume = parser.parse_args()['max_volume'] * 10
	python_settings.preamp_setting.preamp_setting.SetMaxVolume(int(max_volume))
	return jsonify({'max_volume': max_volume})

class UpdateAvailable(Resource):
    @LockCFunction
    def get(self):
        """
        Get if an update is available 
        ---
        tags:
          - Update
        responses:
          200:
            description: Returns a boolean of if an update is available
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        update = python_settings.preamp_setting.get_update_available()
        return {'update_available': update}

    @LockCFunction
    def put(self):
        """
        Start an available update
        ---
        tags:
          - Update
        responses:
          200:
            desciprion: Starts an update returns whether the update was successful
        """
        update = python_settings.preamp_setting.start_update()
        return jsonify({'updated': update})


class UpdateProgress(Resource):

    @LockCFunction
    def get(self):
        """
        The progress of an update in progress
        ---
        tags:
          - Update
        responses:
          200:
            description: Returns a boolean of if an update is available
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        update_progress = python_settings.preamp_setting.preamp_setting.GetUpdatePercent()
        return {'update_progress': update_progress}

class UpdateStatus(Resource):

    @LockCFunction
    def get(self):
        """
        The current step of an upgrade
        ---
        tags:
          - Update
        responses:
          200:
            description: Returns a string describing what the update is doing
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        update_status = python_settings.preamp_setting.preamp_setting.GetUpdateStatus()
        return {'update_status': update_status}

class UpdateInProgress(Resource):

    @LockCFunction
    def get(self):
        """
        Whether the amp is currently updating
        ---
        tags:
          - Update
        responses:
          200:
            description: Returns a boolean of if an update is in progress
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        update = python_settings.preamp_setting.preamp_setting.GetUpdateInProgress()
        return {'update_in_progress': update}



class OutputDevice(Resource):
    @LockCFunction
    def get(self):
        """
	Get the output device 
        
        ---
        tags:
          - Output
        responses:
          200:
            description: Returns the set output volume 
        """
	return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        output_json = {'output_device': []} 
	for i in range(3):
	    output_device = python_settings.preamp_setting.preamp_setting.GetOutputDevice(i)
            output_json['output_device'].append( {'name' : output_device.name,
					    'volume': output_device.volume*0.1,
                                            'mute': output_device.mute,
                                            'polarity': output_device.polarity,
                                            'polarity_temp': output_device.polarity_temp,
                                            'index': i+1
                                            })

	return output_json

    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/output_vol_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('index', type=int, location='args')
	parser.add_argument('name', type=str, default=None, location='args')
	parser.add_argument('volume', type=float, default=None, location='args')
	parser.add_argument('polarity', type=str, default=None, location='args')
	parser.add_argument('polarity_temp', type=str, default=None, location='args')
	parser.add_argument('mute', type=str, default=None, location='args')
	args = parser.parse_args()
        if not args['polarity'] is None:
            args['polarity'] = True if args['polarity'].lower()=='true' else False
        if not args['polarity_temp'] is None:
            args['polarity_temp'] = True if args['polarity_temp'].lower()=='true' else False
        if not args['mute'] is None:
            args['mute'] = True if args['mute'].lower()=='true' else False

        output_device = python_settings.preamp_setting.preamp_setting.GetOutputDevice(args['index']-1)
        for attr in args:
            if args[attr] is None:
                args[attr] = getattr(output_device, attr)
                if attr == 'volume':
                    args[attr] *= 0.1
	python_settings.preamp_setting.preamp_setting.SetOutputDevice(args['name'], int(args['index']-1), int(args['volume']*10), args['mute'], args['polarity'] , args['polarity_temp'])

        return jsonify({'output_volume': {'index': args['index'], 'name': args['name'], 'volume': args['volume'], 'mute': args['mute'], 'polarity': args['polarity'], 'polarity_temp': args['polarity_temp']}})


class Logs(Resource):
    @LockCFunction
    def put(self):
        """
        Upload the current logs of the amplifier
        ---
        tags:
          - Logs
        responses:
          200:
            description: Upload the current logs 
        """
        amp.uploadLogFiles()
        return jsonify({'upload': True})

class Version(Resource):
    @LockCFunction
    def get(self):
        """
        Get current sofware version 
        ---
        tags:
          - Version 
        responses:
          200:
            description:  Output software version
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        version = amp.GetSoftwareVersion()
        return {'software_version': version}


class Mute(Resource):
    @LockCFunction
    def get(self):
        """
        Get if the preamplifier is muted 
        ---
        tags:
          - Global
        responses:
          200:
            description: Returns a boolean of if the device is muted
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        mute = python_settings.preamp_setting.preamp_setting.GetMute()
        return {'mute': mute}


    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/mute_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('mute', type=str, location='args')
	args = parser.parse_args()
        mute = True if args['mute'].lower()=='true' else False
        print "rest mute: ", mute, " args: ", args
	python_settings.preamp_setting.preamp_setting.SetMute(mute)
        return jsonify({'mute': args['mute']})

class Soft(Resource):
    @LockCFunction
    def get(self):
        """
        Get if the preamplifier is soft muted 
        ---
        tags:
          - Global
        responses:
          200:
            description: Returns a boolean of if the device is soft muted
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        soft = python_settings.preamp_setting.preamp_setting.GetSoft()
        return {'soft': soft}


    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/soft_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('soft', type=str, location='args')
	args = parser.parse_args()
        soft = True if args['soft'].lower()=='true' else False
	python_settings.preamp_setting.preamp_setting.SetSoft(soft)
        return jsonify({'soft': args['soft']})

class SoftLevel(Resource):
    @LockCFunction
    def get(self):
        """
        Get if the preamplifier soft level
        ---
        tags:
          - Global
        responses:
          200:
            description: Returns a boolean of if the device is soft muted
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        soft_level = python_settings.preamp_setting.preamp_setting.GetSoftLevel() *0.1
        return {'soft_level': soft_level}


    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/soft_level_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('soft_level', type=float, location='args')
	args = parser.parse_args()
        soft_level = args.soft_level
	python_settings.preamp_setting.preamp_setting.SetSoftLevel(int(soft_level*10))
        return jsonify({'soft_level': soft_level})

class Polarity(Resource):
    @LockCFunction
    def get(self):
        """
        Get if the polarity is positive or negative 
        ---
        tags:
          - Global
        responses:
          200:
            description: Returns a boolean of if the device is soft muted
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        polarity = python_settings.preamp_setting.preamp_setting.GetPolarity()
        return {'polarity': polarity}


    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/polarity_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('polarity', type=str, location='args')
	args = parser.parse_args()
        polarity = True if args['polarity'].lower()=='true' else False
	python_settings.preamp_setting.preamp_setting.SetPolarity(polarity)
        return jsonify({'polarity': args['polarity']})

class PolarityRight(Resource):
    @LockCFunction
    def get(self):
        """
        Get if the polarity right is positive or negative 
        ---
        tags:
          - Global
        responses:
          200:
            description: Returns a boolean of if the device is soft muted
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        polarity_right = python_settings.preamp_setting.preamp_setting.GetPolarityRight()
        return {'polarity_right': polarity_right}


    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/polarity_right_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('polarity', type=str, location='args')
	args = parser.parse_args()
        polarity = True if args['polarity'].lower()=='true' else False
	python_settings.preamp_setting.preamp_setting.SetPolarityRight(polarity)
        return jsonify({'polarity_right': args['polarity']})

class PolarityAux(Resource):
    @LockCFunction
    def get(self):
        """
        Get if the polarity of the auxilary input is positive or negative 
        ---
        tags:
          - Global
        responses:
          200:
            description: Returns a boolean of if the device is soft muted
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        polarity_aux = python_settings.preamp_setting.preamp_setting.GetPolarityAux()
        return {'polarity_aux': polarity_aux}


    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/polarity_aux_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('polarity_aux', type=str, location='args')
	args = parser.parse_args()
        polarity_aux = True if args['polarity_aux'].lower()=='true' else False
	python_settings.preamp_setting.preamp_setting.SetPolarityAux(polarity_aux)
        return jsonify({'polarity_aux': args['polarity_aux']})

class AuxFollowsMain(Resource):
    @LockCFunction
    def get(self):
        """
        Get if the  auxilary input follows the main input 
        ---
        tags:
          - Global
        responses:
          200:
            description: Returns a boolean of if aux follows main is set
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        follows = python_settings.preamp_setting.preamp_setting.GetAuxFollowsMain()
        return {'aux_follows_main': follows}


    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/aux_follows_main_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('follows', type=str, location='args')
	args = parser.parse_args()
        follows = True if args['follows'].lower()=='true' else False
	python_settings.preamp_setting.preamp_setting.SetAuxFollowsMain(follows)
        return jsonify({'follows': args['follows']})

class DisplayBrightness(Resource):
    @LockCFunction
    def get(self):
        """
        Get the display brightness
        ---
        tags:
          - Global
        responses:
          200:
            description: Returns the brightness of the display
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        brightness = python_settings.preamp_setting.preamp_setting.GetDisplayBrightness()
        return {'display_brightness': brightness}


    @LockCFunction
    def post(self):
        """
        Change the master volume, takes number in decibells
        ---
        swagger_from_file: website/swagger/display_brightness_post.yml
        """
	parser = reqparse.RequestParser()
	parser.add_argument('display_brightness', type=int, location='args')
	args = parser.parse_args()
	python_settings.preamp_setting.preamp_setting.SetDisplayBrightness(args['display_brightness'])
        return jsonify({'display_brightness': args['display_brightness']})

#cannot be called Standby for some reason
class UnitStandby(Resource):
    @LockCFunction
    def get(self):
       """
       Get if the unit is in standby
       ---
       tags:
         - Global
       responses:
         200:
           description: Returns a boolean of if the device is in standby 
       """
       return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
       standby = python_settings.preamp_setting.preamp_setting.GetStandby()
       return {'standby': standby}


    @LockCFunction
    def put(self):
        """
        Toggle the stanby state of the amplifier. 
        ---
        tags:
          - Global
        responses:
          200:
            description: Returns the new power state of the preamplifier True for On, False for Off
        """
        standby = python_settings.preamp_setting.preamp_setting.GetStandby()
        python_settings.preamp_setting.preamp_setting.SetStandby(not standby)
        return jsonify({'standby':  not standby})

class ResetInputNames(Resource):
    @LockCFunction
    def post(self):
        """
        Reset the input names. 
        ---
        tags:
          - Global
        responses:
          200:
            description: Changes the input names to their original names 
        """
        python_settings.preamp_setting.preamp_setting.ResetInputNames()
        return jsonify({'reset_input_names':  True})

class ResetOutputNames(Resource):
    @LockCFunction
    def post(self):
        """
        Reset the output names. 
        ---
        tags:
          - Global
        responses:
          200:
            description: Changes the output names to their original names 
        """
        python_settings.preamp_setting.preamp_setting.ResetOutputNames()
        return jsonify({'reset_output_names':  True})

class RestoreFactoryDefaults(Resource):
    @LockCFunction
    def post(self):
        """
        Changes all settings back to the factory defaults 
        ---
        tags:
          - Global
        responses:
          200:
            description: Changes all settings back to the factory defaults
        """
        python_settings.preamp_setting.preamp_setting.RestoreFactoryDefaults()
        return jsonify({'restore_factory_defaults':  True})

class DeviceTable(Resource):
    @LockCFunction
    def get(self):
       """
       Get the Device table
       ---
       tags:
         - Global
       responses:
         200:
           description: Returns a dict of Boulder Amplifier devices that share a room with this one 
       """
       return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
       devices = amp.RoomDeviceMap()
       python_settings.preamp_setting.preamp_setting.GetRoomDevices(devices)
       return _marshal_device_table(devices)

class AmpErrors(Resource):
    @LockCFunction
    def get(self):
        """
       Get the errors from amplifiers in the same room
       ---
       tags:
         - Global
       responses:
         200:
           description: Returns a dict of  errors given by amplifiers
        """
        return jsonify(self.get_dict())

    @LockCFunction
    def get_dict(self):
        rtn = {}
        errors = amp.AmpErrorMap()
        amplifiers = amp.AmplifierMap()
        python_settings.preamp_setting.preamp_setting.GetAmplifiers(amplifiers)
        for i in amplifiers:
            uuid = i.key()
            amplifier = i.data()
            display_name = amplifier.GetDisplayName()
            if display_name == "":
                continue
            rtn[display_name] = {}
            errors = i.data().GetErrors()
            for j in errors:
                error_name = j.key()
                rtn[display_name][error_name] = {}
                for index, k in enumerate(j.data()):
                    rtn[display_name][error_name][index] = {}
                    rtn[display_name][error_name][index]['count'] = k.first
                    rtn[display_name][error_name][index]['time'] = k.second
        return {'amplifier_errors':rtn}

class SerialNumber(Resource):
    @LockCFunction
    def get(self):
        """
       Get the unit's serial number
       ---
       tags:
         - Global
       responses:
         200:
           description: Returns the serial number
        """
        return jsonify(self.get_dict())


    @LockCFunction
    def get_dict(self):
        serial_num = amp.GetSerialNumber()
        return {'serial_number': serial_num}


class All(Resource):
    @LockCFunction
    def get(self):
        """
        Get if the unit is in standby
        ---
        tags:
          - Global
        responses:
          200:
            description: Returns a boolean of if the device is soft muted
        """
        ret = {}
        for endpoint_class, url, kwargs in api.resources:
            if endpoint_class==All:
                continue
            endpoint = endpoint_class()
            if hasattr(endpoint, 'get_dict'):
                #print endpoint
                data = endpoint.get_dict()
                ret.update(data)
        return jsonify({'all': ret})

@api_point.route("/spec")
def spec():
    swag= swagger(current_app, from_file_keyword='swagger_from_file')
    swag['info']['title'] = 'BoulderAmplifier'
    return jsonify(swag)


def init_rest_api(app):
    api.add_resource(RoomName, '/api/room/room_name')
    api.add_resource(UnitStandby, '/api/standby/standby')
    api.add_resource(UpdateAvailable, '/api/update/update_available')
    api.add_resource(UpdateInProgress, '/api/update/update_in_progress')
    api.add_resource(UpdateProgress, '/api/update/update_progress')
    api.add_resource(UpdateStatus, '/api/update/update_status')
    api.add_resource(Balance, '/api/balance/balance')
    api.add_resource(Blend, '/api/blend/blend')
    api.add_resource(MasterVolume, '/api/master_volume/master_volume')
    api.add_resource(DefaultVolume, '/api/master_volume/default_volume')
    api.add_resource(MaxVolume, '/api/master_volume/max_volume')
    api.add_resource(VolumeResolution, '/api/master_volume/volume_resolution')
    api.add_resource(Input, '/api/input/input_device')
    api.add_resource(CurrentInput, '/api/input/current_input_device')
    api.add_resource(CurrentAux, '/api/input/current_aux_device')
    api.add_resource(AuxDisabled, '/api/input/aux_disabled')
    api.add_resource(InputDisabled, '/api/input/input_disabled')
    api.add_resource(OutputDevice, '/api/output/output_device')
    api.add_resource(Logs, '/api/log/upload')
    api.add_resource(Version, '/api/version/version')
    api.add_resource(Mute, '/api/mute/mute')
    api.add_resource(Soft, '/api/soft/soft')
    api.add_resource(SoftLevel, '/api/soft/soft_level')
    api.add_resource(Polarity, '/api/polarity/polarity')
    api.add_resource(PolarityRight, '/api/polarity/polarity_right')
    api.add_resource(PolarityAux, '/api/polarity/polarity_aux')
    api.add_resource(AuxFollowsMain, '/api/aux/aux_follows_main')
    api.add_resource(DisplayBrightness, '/api/display/display_brightness')
    api.add_resource(DeviceTable, '/api/device_table/device_table')
    api.add_resource(ResetInputNames, '/api/input/reset_input_names')
    api.add_resource(ResetOutputNames, '/api/input/reset_output_names')
    api.add_resource(RestoreFactoryDefaults, '/api/global/restore_factory_defaults')
    api.add_resource(AmpErrors, '/api/global/amplifier_errors')
    api.add_resource(SerialNumber, '/api/global/serial_number')
    api.add_resource(All, '/api/all/all')
    api.init_app(app)

def register_headers(app):
    @app.after_request
    def after_request(response):
        response.headers.add('Access-Control-Allow-Origin','*')
        response.headers.add('Access-Control-Allow-Headers', "Authorization, Content-Type")
        response.headers.add('Access-Control-Expose-Headers', "Authorization")
        response.headers.add('Access-Control-Allow-Methods', "GET, POST, PUT, DELETE, OPTIONS")
        response.headers.add('Access-Control-Allow-Credentials', "true")
        response.headers.add('Access-Control-Max-Age', 60 * 60 * 24 * 20)
        return response


if __name__ == '__main__':
    app = Flask(__name__)
    app.register_blueprint(api_point, url_prefix='/api')
    init_rest_api(app)
    register_headers(app)
    app.run(debug=True)
